# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""

This module contains the function necessary to close a window.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gc import collect

from pygtk import require
require("2.0")
from gobject import timeout_add
from gtk import main_quit

from cursor import cursor_position
from position import update_metadata_database
from timeout import quit_mainloop
from internationalization import msg0025


def close_window(editor):
	u"""
	If necessary save the contents of the editors window and close it.

	@param editor: An editor instance
	@type editor: An editor object

	"""

	# The value of maximize_flag value can be lost during a saving operation.
	# Make the value persistent during save operations.
	if editor.maximize_flag:

		editor.persistent_maximize_flag = True

	editor.cursor = cursor_position(editor)
	rectangle = editor.window.get_frame_extents()
	editor.x, editor.y = rectangle.x, rectangle.y
	editor.width, editor.height = editor.get_size()
	editor.hide()

	# Get the window state and position of the file. If the file has not been
	# saved, save it.
	if editor.dirty:

		# Save the file if it has not been saved.
		if editor.filename == msg0025 or not editor.save_flag:

			editor.actiongroup.get_action(u"save_file").activate()

		# Saving is an asynchronous process. So check if the process is
		# completed before closing/quiting the application. If the process is
		# not completed, wait a few seconds, then try to close/quit the
		# application again.
		if editor.saving_flag:

			# Call this function again.
			timeout_add(3000, quit_mainloop, editor)

			return

		# Get the position and state of the editor window before closing the
		# file and store it in the editor's metadata database.
		# See position.py for implementation details of this function.
#		if not editor.saving_flag:

		try:

			editor.maximize_flag = editor.persistent_maximize_flag
			update_metadata_database(editor)

		except AttributeError:

			update_metadata_database(editor)

#	if not editor.saving_flag:

	del editor
	collect()
	main_quit()

	return

