# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Defines an error dialog for Scribes

Scribes only displays fatal errors in message dialogs. Using dialogs to convey
non-fatal error messages should be avoided at all cost.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from pygtk import require
require("2.0")
from gtk import MessageDialog, MESSAGE_ERROR, WIN_POS_CENTER_ON_PARENT
from gtk import STOCK_CLOSE, RESPONSE_CLOSE
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG

from internationalization import msg0133, msg0189
from feedback import update_status_info


def error_dialog(editor, message):
	u"""
	Show an error dialog.

	Displays a dialog containing a non-technical and brief message about an
	exceptional event -- error.

	@param parent: editor
	@type parent: GTK Window
	@param message: an error message
	@type message: string

	@return: dialog with an error message
	@rtype: GTK Dialog
	"""

	primary_msg = "File: " + editor.filename
	secondary_msg = message
	status_msg = msg0189 % editor.filename
	update_status_info(editor, status_msg, "error", False)
	dialog = MessageDialog(type=MESSAGE_ERROR, message_format=primary_msg)

	# Set window properties
	dialog.set_title(msg0133)
	dialog.set_role("error_message_dialog")
	dialog.activate_default()
	dialog.set_transient_for(editor)
	dialog.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
	dialog.set_modal(True)
	dialog.set_skip_taskbar_hint(True)
	dialog.set_icon_name("stock_dialog-error")
	dialog.set_position(WIN_POS_CENTER_ON_PARENT)
	dialog.set_keep_above(True)

	# Set dialog properties
	dialog.add_button(STOCK_CLOSE, RESPONSE_CLOSE)
	dialog.format_secondary_markup(secondary_msg)
	response = dialog.run()

	if response:

		update_status_info(editor, "", "")
		dialog.hide_all()
		dialog.destroy()

	return dialog

