# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Feedback messages to the statusbar area.

This modules documents functions that control textual and visual messages
displayed in the status area.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")
from gobject import timeout_add, source_remove
from gtk.gdk import beep

from timeout import load_string_timeout, user_action_timeout
from modes import add_mode, remove_mode


def show_status_info(editor, message, timeout=False, time=7000):
	"""
	Push feedback information to the editor's statusbar

	@param editor: Reference to the editor.
	@type editor: A editor object.

	@param message: Feedback to display on the editor's statusbar.
	@type message: A string object.

	@param timeout: Whether or not to activate a timer.
	@type timeout: A boolean object

	@param: The length of time before a timer function is activated in msecs.
	@type: An integer object.

	"""

	if editor.message_id:

		editor.status_one.pop(editor.status_one_id)

	editor.status_one_id = editor.status_one.get_context_id(message)
	editor.status_one.push(editor.status_one_id, message)

	if timeout:

		editor.status_timer = timeout_add(time, load_string_timeout, editor)

	return


def update_status_info(editor, message="", mode="", timer=True):
	"""
	Update the status bar with a GTK+ stock icon and a message.

	@param editor:	Reference to editor instance
	@type editor:		An editor object

	@param message:	Message to send to the statusbar
	@type message:	A string

	@param mode:	stock icon to display in the statusbar
	@type mode: 	A string

	@param sound: Whether or not to make the speaker beep.
	@type sound: A gtk.gdk.beep

	"""

	# Prevent the editor from switching to edit mode when the user types into
	# the buffer.
	editor.mode_edit_flag = True

	if mode:

		add_mode(editor, mode)

	else:

		remove_mode(editor)

	try:

		# The flag indicates that status area is currently busy and should not
		# be updated until this operation is over
		editor.status_busy_flag = True

		# Avoid conflicts with other status messages that may need to use the
		# status area after a certain period of time.
		source_remove(editor.status_timer)

		# Avoid conflicts with other status icon messages that may need to use
		# the status area after a certain period of time.
		source_remove(editor.mode_timer)

	except AttributeError:

		pass

	# Feedback to user via the statusbar
	show_status_info(editor, message, timer)

	if timer:

		# Feeback to user via icon in the statusbar. Yes means the user's action
		# is successful. No means it wasn't.
		editor.mode_timer = timeout_add(7000, user_action_timeout, editor)

	if mode == "no":

		beep()

	return

