#! /usr/bin/env python
# -*- coding: utf-8 -*-


from re import compile, IGNORECASE, UNICODE, escape, findall
from collections import deque
from gc import collect

from pygtk import require
require("2.0")
from pango import WEIGHT_HEAVY
from gtk import TEXT_SEARCH_VISIBLE_ONLY, keysyms, HBox, RESIZE_PARENT, Image
from gtk import STOCK_FIND, ICON_SIZE_BUTTON, Label, Entry, Button
from gtk import STOCK_GO_BACK, STOCK_GO_FORWARD, VSeparator, CheckButton
from gconf import client_get_default

from tooltips import create_tooltips, previous_button_tip, next_button_tip
from tooltips import match_case_button_tip, match_word_button_tip, entry_tip
from utility import create_button, response
from feedback import update_status_info
from internationalization import msg0046, msg0047, msg0048, msg0049, msg0050
from internationalization import msg0051, msg0052, msg0053, msg0054, msg0183
from internationalization import msg0184

"""
No documentation, the search and replace behavior is going to be overhauled for
version 0.2.X. Documentation will be updated then.

@Warning: Danger Zone! Nasty hacks follow!

"""

def search_for_text(editor):
	"""search for text in the editor's buffer"""

	# Connect to the gconf daemon
	client = client_get_default()

	# Retrieve case sensitive flag from gconf
	editor.match_case_flag = client.get_bool(u"/apps/scribes/match_case")

	# Retrieve match word flag from gconf
	editor.match_word_flag = client.get_bool(u"/apps/scribes/match_word")

	# Get search string from findbar's text entry field
	string = escape(editor.entry.get_text())

	if not string:

		editor.next_button.set_sensitive(False)

		return

	# Get the first iterator of the editor's buffer
	start_iter = editor.text_buffer.get_start_iter()
	editor.start_iter = start_iter

	# Get the last iterator of the editor's buffer
	end_iter = editor.text_buffer.get_end_iter()
	editor.end_iter = end_iter
	buffer_text = editor.text_buffer.get_text(start_iter, end_iter)

	# Search for matches
	if not editor.match_case_flag:

		# Make string case insensitive via regular expression
		case_insensitive_string = compile(string, IGNORECASE|UNICODE)
		string = case_insensitive_string
		match_list = findall(string, buffer_text, UNICODE)

	else:

		# String is case sensisitive
		case_sensitive_string = compile(string, UNICODE)
		string = case_sensitive_string
		match_list = findall(string, buffer_text, UNICODE)

	# If no matches are found, make the next button insensitive and exit the
	# function.
	if not match_list:

		# See internationalization.py for the full message of msg0046.
		editor.next_button.set_sensitive(False)
		editor.status_one.push(editor.status_one_id, msg0046)

		# If any text in the buffer is selected remove the selection
		if editor.text_buffer.get_selection_bounds():

			selection = editor.text_buffer.get_selection_bounds()
			editor.text_buffer.select_range(selection[0], selection[0])

		return

	# Create a container to store found matches
	editor.found_matches = deque([])

	# If a highligth tag does not exist, create one for found matches
	if not editor.highlight_flag:

		editor.found_match_tag = editor.text_buffer.create_tag(u"highlight",
													background=u"yellow",
													foreground=u"blue",
													weight=WEIGHT_HEAVY)

		editor.highlight_flag = True

	# Whether or not we want to match the entire word as a complete string, or
	# as part of a word in the string
	if not editor.match_word_flag:

		# Matches may or may not be complete words. They may be part of a word.
		# Count for statusbar information
		count = 0

		# Search for the strings in the buffer based on the mathes in the
		# container we created and highlight them
		for string in match_list:

			response()
			count += 1
			found = start_iter.forward_search(string,
											TEXT_SEARCH_VISIBLE_ONLY,
												None)
			editor.found_matches.append(found)
			editor.text_buffer.apply_tag(editor.found_match_tag, found[0], found[1])
			start_iter = found[1]

	else:

		# Mathes are complete words and not part of a word.
		# Search for strings in the buffer based on the matches in the container
		# we created.
		for string in match_list:

			response()
			found = start_iter.forward_search(string,
											TEXT_SEARCH_VISIBLE_ONLY,
												None)
			editor.found_matches.append(found)
			start_iter = found[1]

		# Create a container that will contain matches that are complete words,
		# as opposed to strings that are part of a word.
		real_matches = deque([])

		# Count for status bar information
		count = 0

		# Filter out matches that are complete words from the main container
		for match in editor.found_matches:

			response()

			if match[0].starts_word() and match[1].ends_word():

				count += 1
				real_matches.append(match)
				editor.text_buffer.apply_tag(editor.found_match_tag, match[0],
										match[1])

		editor.found_matches = real_matches

	# Status bar preparations
	editor.status_one.pop(editor.status_one_id)

	# If no matches are found, exit the function
	if count == 0:

		# See internationalization.py for the full message of msg0046
		editor.next_button.set_sensitive(False)
		editor.status_one.push(editor.status_one_id, msg0046)

		# If any text in the buffer is selected remove the selection
		if editor.text_buffer.get_selection_bounds():

			selection = editor.text_buffer.get_selection_bounds()
			editor.text_buffer.select_range(selection[0], selection[0])

		return

	# Show the number of matches found in the statusbar. (One match)
	if count == 1:

		# See internationalization.py for the full message of msg0047
		editor.next_button.set_sensitive(False)
		editor.status_one.push(editor.status_one_id, msg0047 % count)

	# Show the number of mathes found in the statusbar. (More than one matches)
	elif count > 1:

		# See internationalization.py for the full message of msg0048
		editor.status_one.push(editor.status_one_id, msg0048 % count)

	# If this is the first search for the string, follow this path. We do this
	# to keep our search index counter correct.
	if editor.first_search_flag:

		editor.match = editor.found_matches[editor.index]
		editor.text_buffer.select_range(editor.match[0], editor.match[1])

		# Scroll to the found match(es)
		cursor_mark = editor.text_buffer.get_insert()
		editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)
		editor.first_search_flag = False

		if editor.found_matches and editor.replacebar_flag:

			if len(editor.found_matches) > 1:

				editor.replaceall_button.set_sensitive(True)

			editor.replace_button.set_sensitive(True)
			editor.replace_entry.set_sensitive(True)

		return

	# This index keeps track of the found matches in the container
	editor.index += 1

	try:

		editor.match = editor.found_matches[editor.index]
		editor.text_buffer.select_range(editor.match[0], editor.match[1])

	except IndexError:

		editor.next_button.set_sensitive(False)
		editor.index = len(editor.found_matches) - 1

		return

	# Scroll to the found match(es)
	cursor_mark = editor.text_buffer.get_insert()
	editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)

	# Ensure the findbar's next button is insensitive when no more matches are
	# found
	if editor.match == editor.found_matches[-1]:

		editor.next_button.set_sensitive(False)

	if editor.found_matches and editor.replacebar_flag:

		if len(editor.found_matches) > 1:

			editor.replaceall_button.set_sensitive(True)

		editor.replace_button.set_sensitive(True)
		editor.replace_entry.set_sensitive(True)

	if editor.index < 1:

		editor.previous_button.set_sensitive(False)


def remove_highlight_tag(editor):
	"""
	Remove highlight from found matches in the editor's buffer

	editor -- editor object

	"""

	tag = editor.text_buffer.get_tag_table().lookup(u"highlight")

	if tag:

		for tags in editor.found_matches:

			editor.text_buffer.remove_tag(editor.found_match_tag, tags[0], tags[1])


def remove_tag_from_tag_table(editor):
	"""
	Remove highlight tag from the buffer's tag table

	editor -- editor object

	"""

	tag = editor.text_buffer.get_tag_table().lookup(u"highlight")

	if tag:

		editor.text_buffer.get_tag_table().remove(tag)


def next_button_cb(next_button, editor):
	"""
	Callback to the findbar's next button "clicked" signal.

	The signal is emitted when the mouse button is pressed and released while
	the pointer is over the button.

	next_button -- findbar's next button
	editor -- the editor

	"""

	# Ensure that the next button is sensitive only when searches can be
	# performed.
	if editor.index > 0:

		editor.previous_button.set_sensitive(True)

	# Search for string
	search_for_text(editor)

	# Ensure that the next button is sensitive only when searches can be
	# performed.
	if editor.index > 0:

		editor.previous_button.set_sensitive(True)


def previous_button_cb(previous_button, editor):
	"""Callback to the findbar's previous button "clicked" signal.

	The signal is emitted when the mouse button is pressed and released while
	the pointer is over the button.

	previous_button -- findbar's previous button
	editor -- the editor

	"""

	# Ensure that the previous button is insensitive when backward searches
	# cannot be performed
	if editor.index < 1:

		editor.previous_button.set_sensitive(False)

		return

	# I really do not know why this works. But it works. :-P
	if editor.index == len(editor.found_matches) - 1:

		editor.next_button.set_sensitive(True)

	# This index keeps track of the found matches in the container
	editor.index -= 1

	editor.match = editor.found_matches[editor.index]
	editor.text_buffer.select_range(editor.match[0], editor.match[1])

	# Scroll to the found match(es)
	cursor_mark = editor.text_buffer.get_insert()
	editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)

	# Ensure that the previous button is insensitive when backward searches
	# cannot be performed
	if editor.index <= 0:

		editor.previous_button.set_sensitive(False)


def key_press_event_cb(entry, key_press, editor):
	"""
	Callback to the editor's window "key-press-event"

	The "key-press-event" signal is emitted when the user presses a key on the
	keyboard.

	key_press -- Escape
	editor -- editor window

	"""

	# Listen for Escape key press
	if key_press.keyval == keysyms.Escape:

		hide_findbar(editor)


def changed_cb(entry, editor):
	"""
	Handles callback when input is received from the findbar's text entry
	field

	The "changed" signal is emitted when the contents of the widget have
	changed.

	entry -- findbar's entry widget
	editor -- editor object

	"""

	remove_highlight_tag(editor)
	remove_tag_from_tag_table(editor)

	# Remove selections on text
	if editor.text_buffer.get_selection_bounds():

		selection = editor.text_buffer.get_selection_bounds()
		editor.text_buffer.select_range(selection[0], selection[0])

	# These flags need to be reset when text is entered in the text entry field.
	editor.index = 0
	editor.match = None
	editor.first_search_flag = True
	editor.highlight_flag = False

	# Ensure the buttons on the findbar are sensitive when searches can be
	# made and insensitive otherwise
	editor.next_button.set_sensitive(True)
	editor.previous_button.set_sensitive(False)

	if editor.found_matches and editor.replacebar_flag:

		editor.replace_button.set_sensitive(False)
		editor.replaceall_button.set_sensitive(False)
		editor.replace_entry.set_sensitive(False)


def entry_activate_cb(entry, editor):
	"""
	Callback called when the user hits the enter key while in the text entry

	The "activate" signal is emitted when the entry is activated either by user
	action (pressing the Enter key) or programmatically with the
	gtk.Widget.activate() method

	entry -- findbar's text entry field
	editor -- editor object

	"""

	editor.next_button.activate()


def match_case_button_cb(match_case_button, editor):
	"""
	Handles callback when the findbar's match case check button is toggled

	The "toggled" signal is emitted when the togglebutton state changes either
	programmatically or by user action.

	match_case_button -- findbar's match case checkbutton
	editor -- editor object

	"""

	# Remove highlight from text in the editor's buffer
	remove_highlight_tag(editor)
	remove_tag_from_tag_table(editor)

	# Reset flags when the check button is toggled.
	editor.index = 0
	editor.match = None
	editor.first_search_flag = True
	editor.highlight_flag = False

	# Connect to the gconf daemon
	client = client_get_default()
	new_value = match_case_button.get_active()

	if client.get_bool("/apps/scribes/match_case") != new_value:

		client.set_bool(u"/apps/scribes/match_case", new_value)

	# Ensure the buttons on the findbar are sensitive when searches can be
	# made and insensitive otherwise
	editor.next_button.set_sensitive(True)
	editor.previous_button.set_sensitive(False)


	if editor.found_matches and editor.replacebar_flag:

		editor.replace_button.set_sensitive(False)
		editor.replaceall_button.set_sensitive(False)
		editor.replace_entry.set_sensitive(False)

	# Search for text in the findbar's entry field
	editor.entry.activate()


def match_word_button_cb(match_word_button, editor):
	"""
	Handles callback when the findbar's match word check button is toggled

	The "toggled" signal is emitted when the togglebutton state changes either
	programmatically or by user action.

	match_word_button -- findbar's match case checkbutton
	editor -- editor object

	"""

	# Remove highlight from text in the editor's buffer
	remove_highlight_tag(editor)
	remove_tag_from_tag_table(editor)

	# Reset flags when the check button is toggled.
	editor.index = 0
	editor.match = None
	editor.first_search_flag = True
	editor.highlight_flag = False

	# Connect to the gconf daemon
	client = client_get_default()
	new_value = match_word_button.get_active()

	if client.get_bool(u"/apps/scribes/match_word") != new_value:

		client.set_bool(u"/apps/scribes/match_word", new_value)

	# Ensure the buttons on the findbar are sensitive when searches can be
	# made and insensitive otherwise
	editor.next_button.set_sensitive(True)
	editor.previous_button.set_sensitive(False)

	if editor.found_matches and editor.replacebar_flag:

		editor.replace_button.set_sensitive(False)
		editor.replaceall_button.set_sensitive(False)
		editor.replace_entry.set_sensitive(False)

	# Search for text in the findbar's entry field
	editor.entry.activate()

	return


################################################################################
#
#							Findbar Widget
#
################################################################################


def create_findbar(editor):
	"""
	Create the findbar widget for string searching

	The findbar widget contains a search stock icon, a text entry field that
	users can entry strings they wish to search for, a previous button for
	backward navigation of found matches, a next button for foward navigation of
	found matches, a match case check button to allow users to toggle on or off
	case sensitive searches and a match word check button to allow users to
	toggle on or off partial or full string matches.

	editor -- editor window

	return -- a findbar object

	@note: This function needs to be refactored.

	"""

	# Connect to the gconf daemon
	client = client_get_default()

	# Retrieve case sensitive flag from gconf
	editor.match_case_flag = client.get_bool(u"/apps/scribes/match_case")

	# Retrieve match word flag from gconf
	editor.match_word_flag = client.get_bool(u"/apps/scribes/match_word")

	# Initialize a container holding found matches
	editor.found_matches = deque([])

	# Initialize the index that keeps track of the found matches in the
	# container
	editor.index = 0

	# Initialize the matches found
	editor.match = None

	# Initialize the flag that indicates whether or not a string has been
	# searched before. If true, the string has not previously been searched.
	editor.first_search_flag = True

	# Initialize the highlight tag
	editor.highlight_flag = False

	# Create tooltips for the findbar
	tips = create_tooltips()

	# Create the findbar's main container
	findbar = HBox(False, 0)
	editor.connect(u"key_press_event", key_press_event_cb, editor)

	# Set Container Properties
	findbar.set_resize_mode(RESIZE_PARENT)
	findbar.resize_children()

	# Create the stock find icon and add it to the main container
	image = Image()
	image.set_from_stock(STOCK_FIND, ICON_SIZE_BUTTON)
	findbar.pack_start(image, False, False, 3)

	# Create the find label
	if editor.replacebar_flag:

		# See internationalization.py for the full message of msg0049
		label = Label(msg0049)

	else:

		# See internationalization.py for the full message of msg0050
		label = Label(msg0050)

	label.set_use_underline(True)
	findbar.pack_start(label, False, False, 3)

	# Create the text entry widget
	entry = Entry()
	editor.entry = entry
	entry.connect(u"activate", entry_activate_cb, editor)
	entry.connect(u"changed", changed_cb, editor)
	findbar.pack_start(entry, False, False, 3)
	label.set_mnemonic_widget(editor.entry)
	tips.set_tip(entry, entry_tip, tip_private=None)

	# Create the previous button widget
	previous_button = Button()
	editor.previous_button = previous_button
	previous_button.connect(u"clicked", previous_button_cb, editor)
	findbar.pack_start(previous_button, False, False, 3)

	# See internationalization.py for the full message of msg0051
	hbox = create_button(STOCK_GO_BACK, msg0051)
	previous_button.set_use_stock(True)
	previous_button.add(hbox)
	previous_button.set_sensitive(False)
	previous_button.set_focus_on_click(True)
	tips.set_tip(previous_button, previous_button_tip, tip_private=None)

	# Create the next button widget
	next_button = Button()
	editor.next_button = next_button
	next_button.connect(u"clicked", next_button_cb, editor)
	findbar.pack_start(next_button, False, False, 3)

	# See internationalization.py for the full message of msg0052
	hbox = create_button(STOCK_GO_FORWARD, msg0052)
	next_button.add(hbox)
	next_button.set_focus_on_click(True)
	tips.set_tip(next_button, next_button_tip, tip_private=None)

	# Create a separator
	separator = VSeparator()
	findbar.pack_start(separator, False, False, 3)

	# Create a check button for toggling case sensitivity
	# See internationalization.py for the full message of msg0052
	match_case_button = CheckButton(label=msg0053)
	match_case_button.connect(u"toggled", match_case_button_cb, editor)
	findbar.pack_start(match_case_button, False, False, 3)
	match_case_button.set_active(editor.match_case_flag)
	match_case_button.set_use_underline(True)
	tips.set_tip(match_case_button, match_case_button_tip, tip_private=None)

	# Create a check button for toggling full/partial string matches
	# See internationalization.py for the full message of msg0052
	match_word_button = CheckButton(label=msg0054)
	match_word_button.connect(u"toggled", match_word_button_cb, editor)
	findbar.pack_start(match_word_button, False, False, 3)
	match_word_button.set_active(editor.match_word_flag)
	match_word_button.set_use_underline(True)
	tips.set_tip(match_word_button, match_word_button_tip, tip_private=None)

	# I put this here because the check buttons tend to make it sensitive on
	# initialization.
	next_button.set_sensitive(False)

	return findbar


def show_findbar(editor):
	"""
	Show the findbar and focus the elements within it.

	@param editor: Reference to the text editor's instance.
	@type editor: An editor object.

	@note: See flags.py for more information on findbar_flag.
	@note: See internationalization.py for more information on msg0183.

	"""

	editor.findbar_flag = True

	# Make the text editor's buffer uneditable to prevent possible buffer
	# corruptions.
	editor.text_view.set_editable(False)

	# Make the goto, search and replace buttons on the toolbar insensitive.
	editor.actiongroup.get_action(u"goto_line").set_sensitive(False)
	editor.actiongroup.get_action(u"search_text").set_sensitive(False)
	editor.actiongroup.get_action(u"replace_text").set_sensitive(False)

	# Create, show and focus the findbar right above the statusbar area.
	editor.findbar = create_findbar(editor)
	editor.main_container.pack_start(editor.findbar, False, False, 0)
	editor.main_container.reorder_child(editor.findbar, 2)
	editor.entry.grab_focus()
	editor.findbar.show_all()

	# Feedback to the statusbar area indicating the editor is in search mode.
	update_status_info(editor, msg0183, "find", False)

	return


def hide_findbar(editor):
	"""
	Hide the findbar and focus the text editor's buffer.

	@param editor: Reference to the text editor's instance.
	@type editor: An editor object.

	@note: See flags.py for more information on findbar_flag.
	@note: See internationalization for more information on msg0184.

	"""

	editor.findbar_flag = False

	# Destroy the findbar and focus the text's editor's buffer.
	editor.findbar.hide_all()
	editor.findbar.destroy()
	editor.text_view.grab_focus()

	# Look for highlight tags in the text editor's buffer and remove them.
	tag = editor.text_buffer.get_tag_table().lookup(u"highlight")

	if tag:

		remove_highlight_tag(editor)
		editor.text_view.queue_draw()

	# Make the text editor's buffer editable.
	editor.text_view.set_editable(True)

	# Make the goto and search toolbar buttons sensitive. In readonly mode
	# make the replace toolbar button insensitive.
	if editor.readonly:

		flag = False

	else:

		flag = True

	editor.actiongroup.get_action(u"goto_line").set_sensitive(True)
	editor.actiongroup.get_action(u"search_text").set_sensitive(True)
	editor.actiongroup.get_action(u"replace_text").set_sensitive(flag)

	# Feedback to statusbar indicating the findbar has been closed.
	update_status_info(editor, msg0184, "info")
	collect()

	return

