# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
No Documentation. Much of this module is going to be rewritten.

"""

from gc import collect

from pygtk import require
require("2.0")
from gtk import keysyms, STOCK_FIND_AND_REPLACE, HBox, Image, ICON_SIZE_BUTTON
from gtk import Label, Entry, Button

from tooltips import create_tooltips, replace_entry_tip, replace_button_tips
from tooltips import replaceall_button_tip
from utility import create_button, response
from feedback import update_status_info
from findbar import remove_highlight_tag, remove_tag_from_tag_table
from findbar import show_findbar, hide_findbar
from internationalization import msg0077, msg0078, msg0079, msg0080, msg0081
from internationalization import msg0185, msg0186


def key_press_event_cb(entry, key_press, editor):
	"""Callback to the editor's window "key-press-event"

	The "key-press-event" signal is emitted when the user presses a key on the
	keyboard.

	key_press -- Escape
	editor -- editor window

	"""

	# FIXME: There has to be a better way to do this. I think I saw something
	# in the PyGTK FAQ
	# Listen for Escape key press

	if key_press.keyval == keysyms.Escape:

		hide_replacebar(editor)

	return


def replace_button_cb(replace_button, editor):
	"""Handles callback to the replacebar's replace button "clicked" signal.

	The signal is emitted when the mouse button is pressed and released while
	the pointer is over the button.

	replace_button -- replacebar's replace button
	editor -- editor object

	"""

	search_string = editor.entry.get_text()
	replace_string = editor.replace_entry.get_text()

	remove_highlight_tag(editor)
	remove_tag_from_tag_table(editor)

	if not editor.text_buffer.get_selection_bounds():

		match = editor.found_matches[editor.index]
		editor.text_buffer.select_range(match[0], match[1])


	editor.text_buffer.delete_selection(False, False)
	editor.text_buffer.insert_at_cursor(replace_string)

	# Reset flags after a successful replace.
	editor.index = 0
	editor.match = None
	editor.first_search_flag = True
	editor.highlight_flag = False

	editor.replace_button.set_sensitive(False)
	editor.replaceall_button.set_sensitive(False)
	editor.replace_entry.set_sensitive(False)
	editor.previous_button.set_sensitive(False)
	editor.next_button.set_sensitive(True)
	editor.status_one.pop(editor.status_one_id)
	string = msg0077 % (search_string, replace_string)
	editor.status_one_id = editor.status_one.get_context_id(string)
	editor.status_one.push(editor.status_one_id, string)

	return


def replaceall_button_cb(replaceall_button, editor):
	"""Handles callback to the replacebar's replace all button "clicked" signal.

	The signal is emitted when the mouse button is pressed and released while
	the pointer is over the button.

	replace_button -- replacebar's replace  all button
	editor -- editor object

	"""

	editor.replace_button.set_sensitive(False)
	editor.replaceall_button.set_sensitive(False)
	editor.replace_entry.set_sensitive(False)
	editor.previous_button.set_sensitive(False)

	search_string = editor.entry.get_text()
	replace_string = editor.replace_entry.get_text()

	remove_highlight_tag(editor)
	remove_tag_from_tag_table(editor)

	mark_list = []

	for match in editor.found_matches:

		response()
		mark_one = editor.text_buffer.create_mark(None, match[0], True)
		mark_two = editor.text_buffer.create_mark(None, match[1], True)
		mark_list.append(mark_one)
		mark_list.append(mark_two)

	while True:

		response()

		if mark_list:

			pair_one = editor.text_buffer.get_iter_at_mark(mark_list[0])
			pair_two = editor.text_buffer.get_iter_at_mark(mark_list[1])
			editor.text_buffer.select_range(pair_one, pair_two)
			editor.text_buffer.delete_selection(False, False)
			editor.text_buffer.insert_at_cursor(replace_string)

			del mark_list[0]
			del mark_list[0]

		else:

			break

	# Reset flags after a successful replace.
	editor.index = 0
	editor.match = None
	editor.first_search_flag = True
	editor.highlight_flag = False

	editor.status_one.pop(editor.status_one_id)
	string = msg0078 % (search_string, replace_string)
	editor.status_one_id = editor.status_one.get_context_id(string)
	editor.status_one.push(editor.status_one_id, string)

	return


def create_replacebar(editor):
	"""Create the replacebar widget

	The replacebar widget contains a replace stock icon, a text entry field that
	users can entry strings they want to replace, a replace button to replace
	the currently selected match, and a replace all button to replace all found
	matches. The replace bar appears right above the status bar when activated.
	It is hidden and destroyed when deactivated.

	editor -- editor window

	return -- a findbar object

	"""

	editor = editor

	# Create tooltips for the replacebar
	tips = create_tooltips()

	# Create the replacebar's main container
	replacebar = HBox(False, 0)
	editor.connect(u"key_press_event", key_press_event_cb, editor)

	# Create the stock replace icon and add it to the main container
	image = Image()
	image.set_from_stock(STOCK_FIND_AND_REPLACE, ICON_SIZE_BUTTON)
	replacebar.pack_start(image, False, False, 3)

	# Create the replace label
	label = Label(msg0079)
	label.set_use_underline(True)
	replacebar.pack_start(label, False, False, 3)

	# Create the text entry widget
	replace_entry = Entry()
	editor.replace_entry = replace_entry
	replacebar.pack_start(replace_entry, False, False, 1)
	tips.set_tip(replace_entry, replace_entry_tip, tip_private=None)
	replace_entry.set_sensitive(False)
	label.set_mnemonic_widget(editor.replace_entry)

	# Create the replace button widget
	replace_button = Button()
	editor.replace_button = replace_button
	replace_button.connect(u"clicked", replace_button_cb, editor)
	replacebar.pack_start(replace_button, False, False, 5)
	hbox = create_button(STOCK_FIND_AND_REPLACE, msg0080)
	replace_button.set_use_stock(True)
	replace_button.add(hbox)
	replace_button.set_sensitive(False)
	replace_button.set_focus_on_click(True)
	tips.set_tip(replace_button, replace_button_tips, tip_private=None)

	# Create the replace all button widget
	replaceall_button = Button(msg0081)
	editor.replaceall_button = replaceall_button
	replaceall_button.connect(u"clicked", replaceall_button_cb, editor)
	replacebar.pack_start(replaceall_button, False, False, 2)
	replaceall_button.set_use_underline(True)
	replaceall_button.set_sensitive(False)
	replaceall_button.set_focus_on_click(True)
	tips.set_tip(replaceall_button, replaceall_button_tip, tip_private=None)

	return replacebar


def show_replacebar(editor):
	"""
	Show the replacebar and give focus to the elements within it.

	@param editor: Reference to the text editor's instance.
	@type editor: An editor object.

	@note: See flags.py for more information on replace_flag.
	@note: See internationalization.py for more information on msg0185.

	"""

	editor.replacebar_flag = True

	# Show the findbar right on top of the replace.
	show_findbar(editor)
	editor.replacebar = create_replacebar(editor)
	editor.main_container.pack_start(editor.replacebar, False, False, 0)
	editor.main_container.reorder_child(editor.replacebar, 3)
	editor.replacebar.show_all()

	# Feedback to the statusbar indicating the editor is in replace mode.
	update_status_info(editor, msg0185, "replace", False)

	return


def hide_replacebar(editor):
	"""
	Hide the replacebar and give focus back to the text editor's buffer.

	@param editor: Reference to the text editor's instance.
	@type editor: An editor object.

	@note: See flags.py for more information on replace_flag.
	@note: See internationalization.py for more information on msg0186.

	"""

	editor.replacebar_flag = False

	# Hide the replacebar widget.
	hide_findbar(editor)
	editor.replacebar.hide_all()
	editor.text_view.queue_draw()
	editor.replacebar.destroy()

	# Feedback to the statusbar indicating the editor is in replace mode.
	update_status_info(editor, msg0186, "info")
	collect()

	return

