# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Provide syntax higlighting for the editor's buffer.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")
from gtk.gdk import color_parse
from gtksourceview import SourceLanguagesManager, SourceTagStyle
from gnomevfs import get_mime_type
from gconf import client_get_default

from escape import escape_string
from templates import generate_language_templates
from utility import process_color


################################################################################
#
#							Syntax Highlight
#
################################################################################


def syntax_highlight(editor, filename):
	"""
	Highlight keywords in source code for various programming languages.

	@param editor: editor
	@type editor: an editor object
	@param filename: name of text file to be highlighted
	@type filename: string

	@todo: Provide comprehensive highlight for most common language
	@attention: Highlight code is heavily Python biased
	"""

	client = client_get_default()
	manager = SourceLanguagesManager()

	# Retrieve the mime type of the file. Used to detect which syntax highlight
	# to use.
	mime_type = get_mime_type(filename)

	if mime_type:

		# Get programming language from mime type
		language = manager.get_language_from_mime_type(mime_type)

		# Apply syntax tags
		if language:

			language_name = language.get_name()

			######################################################################
			# FIXME FIXME FIXME:
			# A real horrible hack to overcome localization issues. Why did the
			# GTKSOURCEVIEW developers think it was a good idea to localize API
			# names?
			######################################################################
			if language_name.startswith("gettext") or language_name.endswith("gettext"):

				language_name = "gettext-translation"

			if client.dir_exists("/apps/scribes/syntax/" + escape_string(language_name)):

				test = client.all_entries("/apps/scribes/syntax/" +
											escape_string(language_name))

				for entries in test:

					key = (entries.key).replace("/apps/scribes/syntax/"
												+escape_string(language_name)+"/",
												"")
					color_list = []

					try:

						value_list = entries.value.get_list()

					except AttributeError:

						continue

					for items in value_list:

						color_list.append(items.to_string())

					style = SourceTagStyle()
					color = [color_list[0], color_list[1], color_list[2],
							color_list[3]]
					style.foreground = process_color(color)

					if color_list[4] == "True":

						value = True

					else:

						value = False

					style.bold = value

					if color_list[5] == "True":

						value = True

					else:

						value = False

					style.italic = value
					language.set_tag_style(key, style)

			# Activate syntax highlight
			editor.text_buffer.set_highlight(True)
			editor.text_buffer.set_language(language)
			editor.language = language_name
			generate_language_templates(editor)

		else:

			editor.text_buffer.set_highlight(False)

	else:

		editor.text_buffer.set_highlight(False)

	return

