# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module contains functions that interact with the templates database.

The module defines functions to add and remove entries to and from the database,
as well as open and close the database.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from os import path, makedirs
from shelve import open as shelve_open


def open_template_database(editor):
	"""
	Open the template's database for reading.

	"""

	if not path.exists(editor.SCRIBESINFODIR):

		makedirs(editor.SCRIBESINFODIR)

	database = shelve_open(editor.SCRIBESINFODIR + "/templates.gdb", "c")

	return database


def close_template_database(database):

	database.close()

	return


def add_entry(editor, key, value):

	database = open_template_database(editor)
	database[key] = value
	close_template_database(database)

	return


def remove_entry(editor, key):

	database = open_template_database(editor)
	del database[key]
	close_template_database(database)

	return

