# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Toolbar object for the editor created via uimanager

"""

from pygtk import require
require("2.0")
from gtk import UIManager, RESIZE_PARENT, TOOLBAR_ICONS, STOCK_PREFERENCES
from gtk import MenuToolButton, Menu, ImageMenuItem, STOCK_SELECT_COLOR
from gtk import STOCK_PROPERTIES, MenuItem
from utility import create_button, find_toolbar_folder
from tooltips import pref_button_tip, arrow_button_tip, color_button_tip
from tooltips import template_button_tip


def color_cb(item, editor):

	editor.actiongroup.get_action("show_syntax_editor").activate()

	return False


def template_cb(item, editor):

	editor.actiongroup.get_action("show_template_editor").activate()

	return False


def generate_pref_menu(editor):

	menu = Menu()
	item = MenuItem()
	item.add(create_button(STOCK_SELECT_COLOR, "_Color Editor"))
	item.connect("activate", color_cb, editor)
	editor.tips.set_tip(item, color_button_tip)
	menu.append(item)
	item = MenuItem()
	item.add(create_button(STOCK_PROPERTIES, "_Template Editor"))
	item.connect("activate", template_cb, editor)
	editor.tips.set_tip(item, template_button_tip)
	menu.append(item)
	menu.show_all()

	return menu

def clicked_cb(toolbutton, editor):

	editor.actiongroup.get_action("preferences").activate()

	return False


def create_preferences_toolitem(editor):
	"""
	Create a MenuToolButton item.

	The main button launches the preferences editor. The drop down menu provides
	access to the syntax color editor.

	"""

	toolbutton = MenuToolButton(STOCK_PREFERENCES)
	toolbutton.connect("clicked", clicked_cb, editor)
	toolbutton.set_menu(generate_pref_menu(editor))

	return toolbutton


################################################################################
#
#							UIManager Callback
#
################################################################################


def uimanager_cb(uimanager, widget, editor):
	"""
	Handles callback when toolbar is updated

	@param uimanager:	uimanager object
	@type uimanager:	uimanager object
	@param widget:		toolbar object
	@type widget:		toolbar object
	@param editor:		Reference to the editor instance
	@type editor:			An editor object

	"""

	editor.uimanager.ensure_update()

	return


################################################################################
#
#								Toolbar
#
################################################################################


def create_toolbar(editor):
	"""
	Create a toolbar object

	@param editor:	Reference to the editor instance
	@type editor:	An editor object

	@return:		A toolbar object
	@rtype:			A GTK+ Toolbar

	"""

	toolbutton = create_preferences_toolitem(editor)
	toolbutton.set_tooltip(editor.tips, pref_button_tip, pref_button_tip)
	toolbutton.set_arrow_tooltip(editor.tips, arrow_button_tip, arrow_button_tip)

	# Create a UIManager instance
	uimanager = UIManager()
	editor.uimanager = uimanager
	uimanager.connect("add-widget", uimanager_cb, editor)

	# Add the actiongroup to the uimanager
	uimanager.insert_action_group(editor.actiongroup, 0)

	# Merge the User Interface
	toolbar_folder = find_toolbar_folder()
	uifile = toolbar_folder + "/scribes_toolbar_ui.xml"
	uimanager.add_ui_from_file(uifile)

	# Create Toolbar
	toolbar = uimanager.get_widget("/toolbar")

	# Set Container Properties
	toolbar.insert(toolbutton, 13)
	toolbar.set_resize_mode(RESIZE_PARENT)
	toolbar.set_style(TOOLBAR_ICONS)


	# Make Undo and Redo Toolbuttons insensitive by default
	editor.actiongroup.get_action("undo_edits").set_sensitive(False)
	editor.actiongroup.get_action("redo_edits").set_sensitive(False)

	return toolbar

